import logging

from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from Products.CMFCore.utils import getToolByName
from collective.easyform.api import get_actions, set_actions
from collective.lineage.utils import enable_childsite
from plone import api
from plone.app.contenttypes.browser.collection import CollectionView
from plone.app.dexterity.behaviors.constrains import ENABLED
from plone.app.dexterity.behaviors.exclfromnav import IExcludeFromNavigation
from plone.app.event.dx.behaviors import IEventContact
from plone.event.interfaces import IEventAccessor
from plone.event.interfaces import IOccurrence
from plone.supermodel import model
from plone.app.textfield.value import RichTextValue
from Products.CMFPlacefulWorkflow.PlacefulWorkflowTool import \
    WorkflowPolicyConfig_id

REGISTRATION_FORM_ID = 'registration'
DOCUMENTS_FORM_ID = 'submit-documents'
REGISTRATION_INTERNATIONAL_FORM_ID = 'registration-international'
REGISTRATION_US_CITIZEN_FORM_ID = 'registration-us-citizen'

ENABLED_REGISTRATION_FORMS_ATTR = 'enabled_registration_forms'
SET_CONFERENCE_WORKFLOW_ATTR = 'set_conference_to_private_'


class IConference(model.Schema):
    model.load('models/conference.xml')


class ConferenceView(CollectionView):

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.data = IEventAccessor(context)

    def __call__(self):
        if IOccurrence.providedBy(self.context):
            # The transient Occurrence objects cannot be edited. disable the
            # edit border for them.
            self.request.set('disable_border', True)
        return self.index()  # render me.

    def persons(context):
        person_brains = api.content.find(
            portal_type=('person', 'keynoter'),
            context=api.portal.get_navigation_root(context),
            sort_on='sortable_title',
            sort_order='ascending')
        return person_brains


def _set_mailer_adapter_recipient_email(conference, form):
    # set mailer adapter email address to be that of conference owner
    if form:
        actions = get_actions(form)
        mailer = actions['mailer']
        event_contact_aspect = IEventContact(conference)
        conference_contact_email = event_contact_aspect.contact_email
        if mailer.recipient_email != conference_contact_email:
            mailer.recipient_email = conference_contact_email
        # set attendee confirmation mailer sender email and subject line
        if 'attendee_confirmation' in actions:
            attendee_mailer = actions['attendee_confirmation']
            if conference_contact_email:
                attendee_mailer.senderOverride = f"string:{conference_contact_email}"
            else:
                attendee_mailer.senderOverride = None
            attendee_mailer.msg_subject = conference.title
        set_actions(form, actions)


def create_conference_content(conference, event):
    logging.getLogger(__name__).info("setting up the new conference item %s" % conference.getId())

    # make the conference a lineage subsite
    enable_childsite(conference)

    # create conference-specific groups and roles
    conf_organizers_group_name = f"{conference.id}-organizers"
    conf_attendees_group_name = f"{conference.id}-attendees"
    conf_presenters_group_name = f"{conference.id}-presenters"
    api.group.create(groupname=conf_organizers_group_name)
    api.group.create(groupname=conf_attendees_group_name)
    api.group.create(groupname=conf_presenters_group_name)

    # check if private conference, set workflow policy
    _set_conference_workflow_policy(conference)

    # add current user to organizers group
    current_user = api.user.get_current()
    api.group.add_user(groupname=conf_organizers_group_name, username=current_user.getUserName())

    # grant local roles to groups in this conference
    api.group.grant_roles(
        groupname=conf_organizers_group_name, roles=['Owner'], obj=conference)


    # set default collection constraints
    conference.query = [
        {
            'i': 'portal_type',
            'o': 'plone.app.querystring.operation.selection.any',
            'v': ['presentation', 'training_class']
        }
    ]
    conference.sort_on = 'start'
    conference.sort_reversed = False

    # constrain types for conference object
    _constrain_types(conference, ['Folder', 'Document'])

    _create_conference_folder(conference, title='Speakers', allowed_types = ['person', 'keynoter'])
    _create_conference_folder(conference, title='Talks', allowed_types = ['presentation',])
    _create_conference_folder(conference, title='Classes', allowed_types = ['training_class',])
    documents = _create_conference_folder(conference, title='Documents', allowed_types = ['File',])

    # grant Reader to attendees group on the Documents folder
    documents.manage_setLocalRoles(conf_attendees_group_name, ['Reader',])

    # grant organizers group the Editor local role on the conference so members can edit and submit it
    conference.manage_setLocalRoles(conf_organizers_group_name, ['Editor','Contributor','Reader'])

    # copy template forms from site root
    enabled_forms_list = getattr(conference, ENABLED_REGISTRATION_FORMS_ATTR)
    for uid in enabled_forms_list:
        _copy_template_form(conference, uid)
    if 'modify-my-registration' in conference:
        form = conference['modify-my-registration']
        behavior_data = IExcludeFromNavigation(form)
        if behavior_data:
            behavior_data.exclude_from_nav = True
            form.reindexObject()

    # copy template content from site root
    portal = api.portal.get()
    if 'templates' in portal:
        templates = portal['templates']
        path = '/'.join(templates.getPhysicalPath())
        catalog = api.portal.get_tool('portal_catalog')
        template_brains = catalog(
                          path={'query': path, 'depth': 1})
        for item in template_brains:
            id = item.id
            content = templates[id]
            _copy_templates(conference, content)


def _create_conference_folder(container, title, allowed_types):
    new_folder = api.content.create(
        type='Folder',
        title=title,
        container=container)
    _constrain_types(new_folder, allowed_types)
    return new_folder


def _constrain_types(folder, allowed_types):
    aspect = ISelectableConstrainTypes(folder)
    aspect.setConstrainTypesMode(ENABLED)
    aspect.setLocallyAllowedTypes(allowed_types)
    aspect.setImmediatelyAddableTypes(allowed_types)


def _set_conference_workflow_policy(conference):
    logging.getLogger(__name__).info("update conference workflow %s" % conference.getId())
    conf_organizers_group_name = f"{conference.id}-organizers"
    conf_attendees_group_name = f"{conference.id}-attendees"
    conf_presenters_group_name = f"{conference.id}-presenters"
    if getattr(conference, SET_CONFERENCE_WORKFLOW_ATTR, False):
        if getattr(conference, WorkflowPolicyConfig_id, False) is False:
            conference.manage_addProduct['CMFPlacefulWorkflow']\
                .manage_addWorkflowPolicyConfig()
        conf_config = getattr(conference, WorkflowPolicyConfig_id)
        conf_config.setPolicyIn('private-conference', True)
        conf_config.setPolicyBelow('private-conference', True)

        # grant Reviewer to attendees and presenters groups on the Conference folder
        conference.manage_setLocalRoles(conf_attendees_group_name, ['Reviewer',])
        conference.manage_setLocalRoles(conf_presenters_group_name, ['Reviewer',])
    else:
        if getattr(conference, WorkflowPolicyConfig_id, False):
            conf_config = getattr(conference, WorkflowPolicyConfig_id)
            conf_config.setPolicyIn('simple_publication_workflow', True)
            conf_config.setPolicyBelow('simple_publication_workflow', True)

            # remove Reviewer to attendees and presenters groups on the Conference folder
            api.group.revoke_roles(groupname=conf_attendees_group_name,
                                   roles=['Reviewer'],
                                   obj=conference)
            api.group.revoke_roles(groupname=conf_presenters_group_name,
                                   roles=['Reviewer'],
                                   obj=conference)


def _copy_template_form(conference, uid):
    template_form = api.content.get(UID=uid)
    new_form = api.content.copy(source=template_form, target=conference)
    # make sure it's visible
    behavior_data = IExcludeFromNavigation(new_form)
    if behavior_data:
        behavior_data.exclude_from_nav = False
    # set mailer adapter email address to be that of conference owner
    new_form.reindexObject()
    _set_mailer_adapter_recipient_email(conference, new_form)


def _copy_templates(conference, source):
    new_content = api.content.copy(source=source, target=conference)
    # make sure it's visible
    behavior_data = IExcludeFromNavigation(new_content)
    if behavior_data:
        behavior_data.exclude_from_nav = False
    result = get_local_forms(new_content)
    for form in result:
        _set_mailer_adapter_recipient_email(conference,
                                            getattr(new_content, form.id, None))


def update_conference_content(conference, event):
    logging.getLogger(__name__).info("update modified conference %s" % conference.getId())
    result = get_local_forms(conference)
    for form in result:
        _set_mailer_adapter_recipient_email(conference,
                                            getattr(conference, form.id, None))

    # check if private conference, set workflow policy
    _set_conference_workflow_policy(conference)


def get_local_forms(conference):
    catalog = getToolByName(conference, 'portal_catalog')
    conf_path = '/'.join(conference.getPhysicalPath())
    result = catalog(portal_type='EasyForm',
                     path={'query': conf_path, 'depth': 1})
    return result


# Correct Rich Text output from form Generic Setup import
def _update_form_prologue_epilogue(context, form_id):
    portal = api.portal.get()
    form = getattr(portal, form_id, None)
    if form.formPrologue:
        prologue_text = form.formPrologue.raw
        form.formPrologue = RichTextValue(
            raw=prologue_text,
            mimeType='text/html',
            outputMimeType='text/html')
    if form.formEpilogue:
        epilogue_text = form.formEpilogue.raw
        form.formEpilogue = RichTextValue(
            raw=epilogue_text,
            mimeType='text/html',
            outputMimeType='text/html')


def update_forms_richtext(context):
    _update_form_prologue_epilogue(context, REGISTRATION_FORM_ID)
    _update_form_prologue_epilogue(context, DOCUMENTS_FORM_ID)
    _update_form_prologue_epilogue(context, REGISTRATION_INTERNATIONAL_FORM_ID)
    _update_form_prologue_epilogue(context, REGISTRATION_US_CITIZEN_FORM_ID)


def remove_conference_content(conference, event):
    logging.getLogger(__name__).info("remove groups for conference %s"
                                     % conference.getId())
    conf_groups = []
    group_types = ['-organizers', '-attendees', '-presenters']
    for group in group_types:
        conf_groups.append(f"{conference.id}{group}")

    for name in conf_groups:
        try:
            api.group.delete(name)
        except KeyError:
            logging.getLogger(__name__).info("unable to delete %s ,\
                                              group may not exist" % name)
